"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Badges = void 0;
const fs_1 = require("fs");
const utility_marker_1 = require("../utility.marker");
const logger_1 = require("../logger");
function Badges(STATE) {
    return __awaiter(this, void 0, void 0, function* () {
        const CONFIG = yield STATE.getConfigSection('badges');
        if (!CONFIG)
            return;
        const outPath = CONFIG.out || 'README.md';
        const generatedBadges = genBadges(CONFIG);
        const input = utility_marker_1.readFileAndAddMarker(outPath, 'badges');
        yield fs_1.promises.writeFile(outPath, input.replace(utility_marker_1.insertionMarker.regex, utility_marker_1.insertGenerated(generatedBadges, 'badges')));
        logger_1.log('info', logger_1.genMessage('Badges'), outPath);
    });
}
exports.Badges = Badges;
function genBadges(CONFIG) {
    let out = '';
    for (const _badge of CONFIG.badges) {
        out += addBadge(_badge, CONFIG);
    }
    return out.replace(/^ /, '');
}
function addBadge(_badge, CONFIG) {
    const badge = parseBadge(_badge);
    if (Array.isArray(badge)) {
        logger_1.log('info', `Badge: ${badge} is an invalid badge type.`);
        return '';
    }
    const { type, link, params, split, customBadge, customLink } = badge;
    let baseLink = 'https://img.shields.io';
    switch (type) {
        case 'install':
        case 'npmDep':
        case 'npmNode':
        case 'npmLicense':
            baseLink = 'https://badgen.net';
            break;
        case 'codecov':
            baseLink = 'https://codecov.io';
            break;
    }
    if (BADGES[type] || (customBadge && LINKS[link]) || customLink) {
        return ` [![${customBadge ? 'Custom' : type}](${customBadge ? '' : baseLink}${customBadge ? customBadge : replacePlaceholders(CONFIG, BADGES[type])}${params ? params : ''})](${customLink ? customLink : replacePlaceholders(CONFIG, LINKS[link])})`;
    }
    logger_1.log('info', `Badge: "${split.join(' ')}" is not valid.`);
    return '';
}
function parseBadge(badge) {
    if (typeof badge === 'string') {
        let customLink = '';
        if (/ link=.+/.test(badge)) {
            customLink = badge.replace(/.*?link=([^ ]*) ?.*/, '$1');
        }
        let customBadge = '';
        if (/badge=.+/.test(badge)) {
            customBadge = badge.replace(/badge=([^ ]*) ?.*/, '$1');
        }
        const split = badge.split(' ');
        const type = split[0];
        const link = split[1];
        const params = split[2];
        return { type, customBadge, link, customLink, params, split };
    }
    if (typeof badge === 'object' && Array.isArray(badge)) {
        return {
            type: badge[0],
            link: badge[1],
            params: badge[2] || '',
            customBadge: '',
            customLink: '',
            split: badge,
        };
    }
    return [badge];
}
function replacePlaceholders(CONFIG, text) {
    text = text.replace(/<NAME>/g, CONFIG.name);
    text = text.replace(/<GITHUB>/g, CONFIG.github);
    text = text.replace(/<VSCODE>/g, CONFIG.vscode || '');
    text = text.replace(/<REPO>/g, CONFIG.repo);
    return text;
}
const BADGES = {
    circleci: '/circleci/build/github/<GITHUB>/<REPO>',
    codecov: '/gh/<GITHUB>/<REPO>/branch/master/graph/badge.svg',
    vscV: '/visual-studio-marketplace/v/<VSCODE>',
    vscD: '/visual-studio-marketplace/d/<VSCODE>',
    vscI: '/visual-studio-marketplace/i/<VSCODE>',
    vscR: '/visual-studio-marketplace/r/<VSCODE>',
    min: '/bundlephobia/min/<NAME>',
    minzip: '/bundlephobia/minzip/<NAME>',
    install: '/packagephobia/install/<NAME>',
    publish: '/packagephobia/publish/<NAME>',
    npmV: '/npm/v/<NAME>',
    npmDM: '/npm/dm/<NAME>',
    npmDT: '/npm/dt/<NAME>',
    npmDW: '/npm/dw/<NAME>',
    npmDY: '/npm/dy/<NAME>',
    npmDep: '/npm/dependents/<NAME>',
    npmLicense: '/npm/license/<NAME>',
    npmNode: '/npm/node/<NAME>',
    npmTypes: '/npm/types/<NAME>',
    githubFollowers: '/github/followers/<GITHUB>',
    githubForks: '/github/forks/<GITHUB>/<REPO>',
    githubStars: '/github/stars/<GITHUB>/<REPO>',
    githubIssues: '/github/issues/<GITHUB>/<REPO>',
    githubLastCommit: '/github/last-commit/<GITHUB>/<REPO>',
};
const LINKS = {
    npm: 'https://www.npmjs.com/package/<NAME>',
    github: 'https://github.com/<GITHUB>/<REPO>',
    circleci: 'https://app.circleci.com/github/<GITHUB>/<REPO>/pipelines',
    vscode: 'https://marketplace.visualstudio.com/items?itemName=<VSCODE>',
    bundle: 'https://bundlephobia.com/result?p=<NAME>',
    package: 'https://packagephobia.now.sh/result?p=<NAME>',
    codecov: 'https://codecov.io/gh/<GITHUB>/<REPO>',
};
